/**
 * @file Resolvers des groupes
 * @author ofacklam
 */

import {User} from '../users';

import { Announcement } from '../messages/announcements';
import { Event } from '../messages/events';
import { PrivatePost } from '../messages/privatePosts';
import { Question } from '../messages/questions';
import { Answer } from '../messages/answers';

import { Request } from '../requests/requests';
import { UserJoinGroup } from '../requests/userJoinGroup';
import { GroupJoinMetagroup } from '../requests/groupJoinMetagroup';
import { GroupCoauthorEvent } from '../requests/groupCoauthorEvent';

import {Group as LDAP_Group} from '../../../ldap/export/group';
import { userData } from '../../../ldap/export/user';
import { GroupSet } from '../../../utils/tools';
import { DataLoader } from '../../../utils/dataLoader'
import { Context } from '../../typeDefs/queries';

import knex from '../../../../db/knex_router';
import { ApolloError, AuthenticationError } from 'apollo-server-core';


export abstract class Group {

    /**
     * @abstract
     * @memberof GraphQL
     * @class Group
     * @summary Resolvers des groupes
     * @classdesc Une classe abstraite représentant l'interface group du schéma.
     * Comme Apollo Server, par défaut, appelle la propriété / fonction avec le nom a résoudre, il n'y a pas besoin d'écrire les resolvers explicitement.
     * @arg {string} gid - Identifiant du groupe, supposé valide
     * @rights connectedOrOnplatal
     */
    constructor(gid: string) {
        this.gid = gid;
        this.m_loader = new DataLoader(this.fetchData.bind(this));
    }

    /**
     * @memberof GraphQL.Group#
     * @function fetchData
     * @summary Fonction qui va chercher toutes les données sur ce groupe dans le stockage approprié, si ce n'est pas déja fait.
     * @returns {Promise(boolean)} Renvoie true si le chargement est réussi.
     * @async
     * @protected
     * @abstract
     */
    protected abstract async fetchData(): Promise<boolean>

    /**
     * Protected properties.
     * Ce sont tous les champs triviaux, ie qui peuvent etre récupérés en une seule requete a la base (que ce soit pour SimpleGroup ou MetaGroup),
     * ce qui permet d'etre plus efficace.
     * La variable dataLoaded témoigne de si on est déja allés chercher les données.
     * ATTENTION. Ce ne sont PAS directement les resolvers, il FAUT donc qu'ils aient un nom DIFFÉRENT du nom des champs dans le schéma.
     */
    protected m_loader: DataLoader

    protected m_name: string
    protected m_description: string
    protected m_mail: string
    protected m_website: string

    /**
     * Ci-dessous les resolvers a proprement parler.
     */

    __typename : string = "Bad group type";

    /** @rights connectedOrOnplatal */
    gid: string;

    /**
     * @memberof GraphQL.Group#
     * @function name
     * @summary Renvoie le nom du groupe
     * @return {Promise(string)}
     * @rights connectedOrOnplatal
     * @async
     */
    async name(args, context: Context, info): Promise<string> {
        return this.getName();
    }

    async getName(): Promise<string> {
        await this.m_loader.load();
        return this.m_name;
    }

    /**
     * @memberof GraphQL.Group#
     * @function description
     * @summary Renvoie la description du groupe
     * @return {Promise(string)}
     * @rights viewer
     * @async
     */
    async description(args, context: Context, info): Promise<string> {
        if(await context.models.auth.isViewer(this.gid)) {
            return this.getDescription();
        }
        throw new AuthenticationError("Not a viewer");
    }

    async getDescription(): Promise<string> {
        await this.m_loader.load();
        return this.m_description;
    }

    /**
     * @memberof GraphQL.Group#
     * @function mail
     * @summary Renvoie l'adresse mail
     * @return {Promise(string)}
     * @rights viewer
     * @async
     */
    async mail(args, context: Context, info): Promise<string> {
        if (await context.models.auth.isViewer(this.gid)) {
            return this.getMail();
        }
        throw new AuthenticationError("Not a viewer");
    }

    async getMail() : Promise<string> {
        await this.m_loader.load();
        return this.m_mail;
    }

    /**
     * @memberof GraphQL.Group#
     * @function website
     * @summary Renvoie l'adresse du site web
     * @return {Promise(string)}
     * @rights viewer
     * @async
     */
    async website(args, context: Context, info): Promise<string> {
        if (await context.models.auth.isViewer(this.gid)) {
            return this.getWebsite();
        }
        throw new AuthenticationError("Not a viewer");
    }

    async getWebsite() : Promise<string> {
        await this.m_loader.load();
        return this.m_website;
    }

    /**
     * @memberof GraphQL.Group#
     * @function admins
     * @summary Renvoie la liste des admins
     * @return {Promise(User[])}
     * @rights viewer
     * @async
     * @abstract
     */
    abstract async admins(args, context: Context, info): Promise<User[]>

    /**
     * @memberof GraphQL.Group#
     * @function frontPage
     * @summary Renvoie la page principale
     * @return {Promise(string)}
     * @rights viewer
     * @async
     * @abstract
     */
    abstract async frontPage(args, context: Context, info): Promise<string>

    /**
     * @memberof GraphQL.Group#
     * @function questions
     * @summary Renvoie les questions adressées a ce groupe
     * @return {Promise(Question[])}
     * @rights viewer
     * @async
     */
    async questions(args, context: Context, info): Promise<Question[]> {
        if (await context.models.auth.isViewer(this.gid)) {
            return context.models.message.getAllQuestionsReceived(this.gid);

            // let result = await knex('questions').select().whereIn('id', received_messages);
            // for(let entry of result){
            //     entry.type = "Question";
            // }
            // return result;
        }
        throw new AuthenticationError("Not a viewer");
    }

    /**
     * @memberof GraphQL.Group#
     * @function answers
     * @summary Renvoie les réponses de ce groupe
     * @return {Promise(Answer[])}
     * @rights viewer
     * @async
     */
    async answers(args, context: Context, info): Promise<Answer[]> {
        if (await context.models.auth.isViewer(this.gid)) {
            return context.models.message.getAllAnswersSent(this.gid);

            // let received_messages = await selectors.recievedMessages(user, groupUID);
            // let result = await knex('answers').select().whereIn('id', received_messages);
            // for(let entry of result){
            //     entry.type = "Answer";
            // }
            // return result;
        }
        throw new AuthenticationError("Not a viewer");
    }

    /**
     * @memberof GraphQL.Group#
     * @function announcementsToGroup
     * @summary Renvoie les annonces adressées a ce groupe
     * @return {Promise(Announcement[])}
     * @rights viewer
     * @async
     */
    async announcementsToGroup(args, context: Context, info): Promise<Announcement[]> {
        if (await context.models.auth.isViewer(this.gid)) {
            //03/03/19
            let res = new GroupSet();
            res.add(this.gid);
            return context.models.message.getAllAnnouncementsReceived(res);
        }
        throw new AuthenticationError("Not a viewer");
    }

    /**
     * @memberof GraphQL.Group#
     * @function eventsToGroup
     * @summary Renvoie les evenements adressées a ce groupe
     * @return {Promise(Event[])}
     * @rights viewer
     * @async
     */
    async eventsToGroup(args, context: Context, info): Promise<Event[]> {
        if (await context.models.auth.isViewer(this.gid)) {
            //03/03/19
            let res = new GroupSet();
            res.add(this.gid);
            return context.models.message.getAllEventsTo(res);
        }
        throw new AuthenticationError("Not a viewer");
    }

    /**
     * @memberof GraphQL.Group#
     * @function announcementsFromGroup
     * @summary Renvoie les annonces créées par ce groupe
     * @return {Promise(Announcement[])}
     * @rights member
     * @async
     */
    async announcementsFromGroup(args, context: Context, info): Promise<Announcement[]> {
        if(await context.models.auth.isMember(this.gid)) {
            //03/03/19
            let res = new GroupSet();
            res.add(this.gid);
            return context.models.message.getAllAnnouncementsSent(res);
        }
        throw new AuthenticationError("Not a member");
    }

    /**
     * @memberof GraphQL.Group#
     * @function eventsFromGroup
     * @summary Renvoie les evenements créés par ce groupe
     * @return {Promise(Event[])}
     * @rights member
     * @async
     */
    async eventsFromGroup(args, context: Context, info): Promise<Event[]> {
        if (await context.models.auth.isMember(this.gid)) {
            //03/03/19
            let res = new GroupSet();
            res.add(this.gid);
            return context.models.message.getAllEventsFrom(res);
        }
        throw new AuthenticationError("Not a member");
    }

    /**
     * @memberof GraphQL.Group#
     * @function privatePosts
     * @summary Renvoie les posts internes du groupe
     * @return {Promise(PrivatePost[])}
     * @rights member
     * @async
     */
    async privatePosts(args, context: Context, info): Promise<PrivatePost[]> {
        if (await context.models.auth.isMember(this.gid)) {
            //03/03/19
            let res = new GroupSet();
            res.add(this.gid);
            return context.models.message.getAllPrivatePosts(res);
        }
        throw new AuthenticationError("Not a member");
    }

    /**
     * @memberof GraphQL.Group#
     * @function requestsToGroup
     * @summary Renvoie toutes les requetes adressées au groupe.
     * @return {Promise(Request[])}
     * @rights admin
     * @async
     */
    async requestsToGroup(args, context: Context, info): Promise<Request[]> {
        if (await context.models.auth.isAdmin(this.gid)) {
            return context.models.request.getRequestsToGroup(this.gid);
        }
        throw new AuthenticationError("Not an admin");
    }

    /**
     * @memberof GraphQL.Group#
     * @function userJoinGroupRequestsToGroup
     * @summary Renvoie toutes les requetes UserJoinGroup adressées au groupe.
     * @return {Promise(UserJoinGroup[])}
     * @rights admin
     * @async
     */
    async userJoinGroupRequestsToGroup(args, context: Context, info): Promise<UserJoinGroup[]> {
        if (await context.models.auth.isAdmin(this.gid)) {
            return context.models.request.getUserJoinGroupRequestsToGroup(this.gid);
        }
        throw new AuthenticationError("Not an admin");
    }

    /**
     * @memberof GraphQL.Group#
     * @function groupJoinMetagroupRequestsToGroup
     * @summary Renvoie toutes les requetes GroupJoinMetagroup adressées au groupe.
     * @return {Promise(GroupJoinMetagroup[])}
     * @rights admin
     * @async
     */
    async groupJoinMetagroupRequestsToGroup(args, context: Context, info): Promise<GroupJoinMetagroup[]> {
        if (await context.models.auth.isAdmin(this.gid)) {
            return context.models.request.getGroupJoinMetagroupRequestsToGroup(this.gid);
        }
        throw new AuthenticationError("Not an admin");
    }

    /**
     * @memberof GraphQL.Group#
     * @function groupCoauthorEventRequestsToGroup
     * @summary Renvoie toutes les requetes GroupCoauthorEvent adressées au groupe.
     * @return {Promise(GroupCoauthorEvent[])}
     * @rights admin
     * @async
     */
    async groupCoauthorEventRequestsToGroup(args, context: Context, info): Promise<GroupCoauthorEvent[]> {
        if (await context.models.auth.isAdmin(this.gid)) {
            return context.models.request.getGroupCoauthorEventRequestsToGroup(this.gid);
        }
        throw new AuthenticationError("Not an admin");
    }

    /**
     * @memberof GraphQL.Group#
     * @function visibilityEdges
     * @summary Renvoie les groupes auxquels ce groupe est visible.
     * @return {Promise(Group[])}
     * @rights viewer
     * @async
     */
    async visibilityEdges(args, context: Context, info): Promise<Group[]> {
        if (await context.models.auth.isViewer(this.gid)) {
            throw "Not implemented"
        }
        throw new AuthenticationError("Not a viewer");
    }
}

