/**
 * @file Resolvers des méta-groupes
 * @author ofacklam
 */

import { User } from '../users';
import { Group } from './groups';
import { SimpleGroup } from './simpleGroups';

import { Group as LDAP_Group } from '../../../ldap/export/group';
import { userData } from '../../../ldap/export/user';
import { GroupSet } from '../../../utils/tools';
import { DataLoader } from '../../../utils/dataLoader'
import { Context } from '../../typeDefs/queries';

import knex from '../../../../db/knex_router';
import { ApolloError, AuthenticationError } from 'apollo-server-core';


export class MetaGroup extends Group {

    /**
     * @memberof GraphQL
     * @class MetaGroup
     * @extends GraphQL.Group
     * @summary Resolvers des méta-groupes
     * @classdesc Une classe représentant le type MetaGroup du schéma.
     * Comme Apollo Server, par défaut, appelle la propriété / fonction avec le nom a résoudre, il n'y a pas besoin d'écrire les resolvers explicitement.
     * @arg {string} gid - Identifiant du méta-groupe, supposé valide
     * @rights connectedOrOnplatal
     */
    constructor(gid: string) {
        super(gid);
    }

    /**
     * @memberof GraphQL.MetaGroup#
     * @function tryCreate
     * @summary Fonction qui va essayer de créer le méta-groupe correspondant
     * @arg {string} gid - Identifiant du méta-groupe, sans hypothese sur la validité.
     * @returns {Promise(MetaGroup)} - Renvoie le méta-groupe créé, ou null en cas d'erreur.
     * @rights connectedOrOnplatal
     * @async
     * @static
     */
    static async tryCreate(gid: string): Promise<MetaGroup> {
        let g = new MetaGroup(gid);
        if (await g.m_loader.load()) {
            return g;
        }
        return null;
    }

    /**
     * @memberof GraphQL.MetaGroup#
     * @function fetchData
     * @summary Fonction qui va chercher toutes les données sur ce méta-groupe dans la BDD, si ce n'est pas déja fait.
     * @returns {Promise(boolean)} Renvoie true si le chargement est réussi.
     * @async
     * @protected
     */
    protected async fetchData(): Promise<boolean> {
        let data = await knex.select(
            'name',
            'description',
            'mail', 
            'frontPage', 
            'website').from('metagroups').where('gid', this.gid);

        if (data.length > 0) {
            let g = data[0];

            this.m_name = g.name;
            this.m_description = g.description;
            this.m_mail = g.mail;
            this.m_website = g.website;

            this.m_frontPage = g.frontPage;

            return true;
        }
        return false;
    }

    /**
     * Protected properties.
     * Ce sont tous les champs triviaux, ie qui peuvent etre récupérés en une seule requete a la base (que ce soit pour SimpleGroup ou MetaGroup),
     * ce qui permet d'etre plus efficace.
     * La variable dataLoaded témoigne de si on est déja allés chercher les données.
     * ATTENTION. Ce ne sont PAS directement les resolvers, il FAUT donc qu'ils aient un nom DIFFÉRENT du nom des champs dans le schéma.
     */
    protected m_frontPage : string;


    /**
     * Ci-dessous les resolvers a proprement parler.
     */

    __typename: string = "MetaGroup";

    /**
     * @memberof GraphQL.MetaGroup#
     * @function admins
     * @summary Renvoie la liste des admins
     * @return {Promise(User[])}
     * @rights viewer
     * @async
     */
    async admins(args, context: Context, info): Promise<User[]> {
        if (await context.models.auth.isViewer(this.gid)) {
            return context.models.group.getMetaGroupAdmins(this.gid);
        }
        throw new AuthenticationError("Not a viewer");
    }

    /**
     * @memberof GraphQL.MetaGroup#
     * @function members
     * @summary Renvoie la liste des groupes membres
     * @return {Promise(SimpleGroup[])}
     * @rights viewer
     * @async
     */
    async members(args, context: Context, info): Promise<SimpleGroup[]> {
        if (await context.models.auth.isViewer(this.gid)) {
            let arg = await context.models.group.getAllSimpleGroupsFromMeta(this.gid);
            let res = new Array<SimpleGroup>();
            for (let g of arg) {
                res.push(new SimpleGroup(g));
            }
            return res;

            /*let member_group_list = await knex.distinct().select().from('groups')
                .innerJoin('meta_group_membership', 'groups.uid', 'meta_group_membership.member_uid')
                .where('meta_group_membership.union_uid', '=', group.gid);
            let members = new Set;
            for (const memberGroup of member_group_list) {
                let res = await getGroupMemberUsers(quser, new Group(memberGroup));
                for (const member of res.values()) {
                    members.add(member);
                }
            }
            return members;*/
        }
        throw new AuthenticationError("Not a viewer");
    }

    /**
     * @memberof GraphQL.SimpleGroup#
     * @function frontPage
     * @summary Renvoie la page principale
     * @return {Promise(string)}
     * @rights viewer
     * @async
     */
    async frontPage(args, context: Context, info): Promise<string> {
        if (await context.models.auth.isViewer(this.gid)) {
            return this.getFrontPage();
        }
        throw new AuthenticationError("Not a viewer");
    }

    async getFrontPage(): Promise<string> {
        await this.m_loader.load();
        return this.m_frontPage;
    }

}
