/**
 * @file Resolvers des groupes simples
 * @author ofacklam
 */

import { User } from '../users';
import { Group } from './groups';
import { MetaGroup } from './metaGroups';

import { Group as LDAP_Group } from '../../../ldap/export/group';
import { userData } from '../../../ldap/export/user';
import { GroupSet } from '../../../utils/tools';
import { DataLoader } from '../../../utils/dataLoader'
import { Context } from '../../typeDefs/queries';

import knex from '../../../../db/knex_router';
import { ApolloError, AuthenticationError } from 'apollo-server-core';

export class SimpleGroup extends Group {

    /**
     * @memberof GraphQL
     * @class SimpleGroup
     * @extends GraphQL.Group
     * @summary Resolvers des groupes simples
     * @classdesc Une classe représentant le type SimpleGroup du schéma.
     * Comme Apollo Server, par défaut, appelle la propriété / fonction avec le nom a résoudre, il n'y a pas besoin d'écrire les resolvers explicitement.
     * @arg {string} gid - Identifiant du groupe simple, supposé valide.
     * @rights connectedOrOnplatal
     */
    constructor(gid: string) {
        super(gid);
    }

    /**
     * @memberof GraphQL.SimpleGroup#
     * @function tryCreate
     * @summary Fonction qui va essayer de créer le groupe simple correspondant
     * @arg {string} gid - Identifiant du groupe simple, sans hypothese sur la validité.
     * @returns {Promise(SimpleGroup)} - Renvoie le groupe simple créé, ou null en cas d'erreur.
     * @rights connectedOrOnplatal
     * @async
     * @static
     */
    static async tryCreate(gid: string): Promise<SimpleGroup> {
        let g = new SimpleGroup(gid);
        if (await g.m_loader.load()) {
            return g;
        }
        return null;
    }

    /**
     * @memberof GraphQL.SimpleGroup#
     * @function fetchData
     * @summary Fonction qui va chercher toutes les données sur ce groupe simple dans le LDAP, si ce n'est pas déja fait.
     * @returns {Promise(boolean)} Renvoie true si le chargement est réussi.
     * @async
     * @protected
     */
    protected async fetchData(): Promise<boolean> {
        try {
            let data = await LDAP_Group.peek(this.gid);
            if (data.gid !== this.gid) {
                throw "Error in search";
            }

            this.m_name = data.name;
            this.m_description = data.description;
            this.m_mail = data.email;
            this.m_website = data.site;

            this.m_members = data.members;
            this.m_speakers = data.speakers;
            this.m_admins = data.admins;
            this.m_likers = data.followers;

            this.m_parents = data.parents;

            //console.log("Ran SimpleGroup fetchdata() for " + this.gid + " : " + this.m_dataLoaded);
            return true;
        }
        catch {
            return false;
        }
    }

    /**
     * Protected properties.
     * Ce sont tous les champs triviaux, ie qui peuvent etre récupérés en une seule requete a la base (que ce soit pour SimpleGroup ou MetaGroup),
     * ce qui permet d'etre plus efficace.
     * La variable dataLoaded témoigne de si on est déja allés chercher les données.
     * ATTENTION. Ce ne sont PAS directement les resolvers, il FAUT donc qu'ils aient un nom DIFFÉRENT du nom des champs dans le schéma.
     */
    protected m_members: string[]
    protected m_speakers: string[]
    protected m_admins: string[]
    protected m_likers: string[]
    protected m_parents: string[]

    /**
     * Ci-dessous les resolvers a proprement parler.
     */

    __typename: string = "SimpleGroup";

    /**
     * @memberof GraphQL.SimpleGroup#
     * @function members
     * @summary Renvoie la liste des membres
     * @return {Promise(User[])}
     * @rights viewer
     * @async
     */
    async members(args, context: Context, info): Promise<User[]> {
        if (await context.models.auth.isViewer(this.gid)) {
            let listUser = await this.getMembers();
            return Promise.all(listUser.map(uid => context.models.user.getUser(uid)));
        }
        throw new AuthenticationError("Not a viewer");
    }

    async getMembers() : Promise<string[]> {
        await this.m_loader.load();
        return this.m_members;
    }

    /**
     * @memberof GraphQL.SimpleGroup#
     * @function speakers
     * @summary Renvoie la liste des speakers
     * @return {Promise(User[])}
     * @rights viewer
     * @async
     */
    async speakers(args, context: Context, info): Promise<User[]> {
        if (await context.models.auth.isViewer(this.gid)) {
            let listUser = await this.getSpeakers();
            return Promise.all(listUser.map(uid => context.models.user.getUser(uid)));
        }
        throw new AuthenticationError("Not a viewer");
    }

    async getSpeakers(): Promise<string[]> {
        await this.m_loader.load();
        return this.m_speakers;
    }

    /**
     * @memberof GraphQL.SimpleGroup#
     * @function admins
     * @summary Renvoie la liste des admins
     * @return {Promise(User[])}
     * @rights viewer
     * @async
     */
    async admins(args, context: Context, info): Promise<User[]> {
        if (await context.models.auth.isViewer(this.gid)) {
            let listUser = await this.getAdmins();
            return Promise.all(listUser.map(uid => context.models.user.getUser(uid)));
        }
        throw new AuthenticationError("Not a viewer");
    }

    async getAdmins(): Promise<string[]> {
        await this.m_loader.load();
        return this.m_admins;
    }



    /**
     * @memberof GraphQL.SimpleGroup#
     * @function likers
     * @summary Renvoie la liste des sympathisants
     * @return {Promise(User[])}
     * @rights viewer
     * @async
     */
    async likers(args, context: Context, info): Promise<User[]> {
        if (await context.models.auth.isViewer(this.gid)) {
            let listUser = await this.getLikers();
            return Promise.all(listUser.map(uid => context.models.user.getUser(uid)));
        }
        throw new AuthenticationError("Not a viewer");
    }

    async getLikers(): Promise<string[]> {
        await this.m_loader.load();
        return this.m_likers;
    }

    /**
     * @memberof GraphQL.SimpleGroup#
     * @function frontPage
     * @summary Renvoie la page principale
     * @return {Promise(string)}
     * @rights viewer
     * @async
     */
    async frontPage(args, context: Context, info): Promise<string> {
        if (await context.models.auth.isViewer(this.gid)) {
            throw "Not implemented";
        }

        throw new AuthenticationError("Not a viewer");
    }

    /**
     * @memberof GraphQL.SimpleGroup#
     * @function parent
     * @summary Renvoie le groupe parent
     * @return {Promise(SimpleGroup)}
     * @rights viewer
     * @async
     */
    async parents(args, context: Context, info): Promise<SimpleGroup[]> {
        if (await context.models.auth.isViewer(this.gid)) {
            return this.getParents();
        }
        throw new AuthenticationError("Not a viewer");
    }

    async getParents(): Promise<SimpleGroup[]> {
        await this.m_loader.load();
        return this.m_parents.map(gid => {
            return new SimpleGroup(gid);
        });
    }

    /**
     * @memberof GraphQL.SimpleGroup#
     * @function children
     * @summary Renvoie la liste des groupes enfants
     * @return {Promise(SimpleGroup[])}
     * @rights viewer
     * @async
     */
    async children(args, context: Context, info): Promise<SimpleGroup[]> {
        if (await context.models.auth.isViewer(this.gid)) {
            throw "Not implemented";
        }
        throw new AuthenticationError("Not a viewer");
    }

    /**
     * @memberof GraphQL.SimpleGroup#
     * @function memberOfMeta
     * @summary Renvoie la liste des méta-groupes dont ce groupe fait partie
     * @return {Promise(MetaGroup[])}
     * @rights viewer
     * @async
     */
    async memberOfMeta(args, context: Context, info): Promise<MetaGroup[]> {
        if (await context.models.auth.isViewer(this.gid)) {
            let arg = await context.models.group.getAllMetaGroupsFromSimple(this.gid);
            let res = new Array<MetaGroup>();
            for (let g of arg) {
                res.push(new MetaGroup(g));
            }
            return res;
        }
        throw new AuthenticationError("Not a viewer");
    }

    /**
     * @memberof GraphQL.SimpleGroup#
     * @function school
     * @summary Renvoie l'école d'origine
     * @return {Promise(string)}
     * @rights viewer
     * @async
     */
    async school(args, context: Context, info): Promise<string> {
        if (await context.models.auth.isViewer(this.gid)) {
            throw "Not implemented";
        }
        throw new AuthenticationError("Not a viewer");
    }
}
