/**
 * @file Resolvers pour les annonces
 * @author akka, ofacklam
 */

import { Group } from '../groups/groups';
import { SimpleGroup } from "../groups/simpleGroups";
import { MetaGroup } from "../groups/metaGroups";

import { User } from '../users';

import { Message } from './messages';
import { Event } from './events';

import { GroupSet } from '../../../utils/tools';
import { Context } from '../../typeDefs/queries';

import knex from '../../../../db/knex_router';
import { ApolloError } from 'apollo-server-core';

export class Announcement extends Message {

    /**
     * @memberof GraphQL
     * @class Announcement
     * @extends GraphQL.Message
     * @summary Resolvers des announcements
     * @classdesc Une classe représentant le type Announcement du schéma.
     * Comme Apollo Server, par défaut, appelle la propriété / fonction avec le nom a résoudre, il n'y a pas besoin d'écrire les resolvers explicitement.
     * @arg {number} mid - Identifiant du message, supposé valide.
     * @rights membre d'un groupe author ou d'un groupe recipient
     */
    constructor(mid: number) {
        super(mid);
    }

    /**
     * @memberof GraphQL.Announcement#
     * @function tryCreate
     * @summary Fonction qui va essayer de créer l'annonce correspondante
     * @arg {number} mid - Identifiant du message, sans hypothese sur la validité.
     * @returns {Promise(Announcement)} - Renvoie le Announcement créé, ou null en cas d'erreur.
     * @rights membre d'un groupe author ou d'un groupe recipient
     * @async
     * @static
     */
    static async tryCreate(mid: number): Promise<Announcement> {
        let m = new Announcement(mid);
        if (await m.m_loader.load()) {
            return m;
        }
        return null;
    }

    /**
     * @memberof GraphQL.Announcement#
     * @function fetchData
     * @summary Fonction qui va chercher toutes les données sur cet announcement dans la BDD, si ce n'est pas déja fait.
     * @returns {Promise(boolean)} Renvoie true si le chargement est réussi.
     * @async
     * @protected
     */
    protected async fetchData(): Promise<boolean> {
        let data = await knex.select('created_at',
            'updated_at',
            'title',
            'content',
            'views',
            'parent').from('messages_announcements').where('mid', this.mid);

        if (data.length > 0) {
            let m = data[0];

            this.m_createdAt = m.created_at;
            this.m_updatedAt = m.updated_at;
            this.m_title = m.title;
            this.m_content = m.content;
            this.m_views = m.views;
            this.m_parent = m.parent;

            return true;
        }
        
        return false;
    }

    /**
     * Protected properties.
     * Ce sont tous les champs triviaux, ie qui peuvent etre récupérés en une seule requete a la BDD,
     * ce qui permet d'etre plus efficace.
     * La variable dataLoaded témoigne de si on est déja allés chercher les données.
     * ATTENTION. Ce ne sont PAS directement les resolvers, il FAUT donc qu'ils aient un nom DIFFÉRENT du nom des champs dans le schéma.
     */
    protected m_views: number
    protected m_parent: number

    /**
     * Ci-dessous les resolvers a proprement parler.
     */

    __typename: string = "Announcement";

    /**
     * @memberof GraphQL.Announcement#
     * @function authors
     * @summary Renvoie les groupes auteurs
     * @return {Promise(Group[])}
     * @rights membre d'un groupe author ou d'un groupe recipient
     * @async
     */
    async authors(args, context: Context, info): Promise<Group[]> {
        let res = new Array<Group>();

        let groups = await context.models.message.getAnnouncementAuthors(this.mid);
        for(let g of groups) {
            res.push(await context.models.group.getGroup(g));
        }

        return res;
    }

    /**
     * @memberof GraphQL.Announcement#
     * @function recipients
     * @summary Renvoie les groupes destinataires
     * @return {Promise(Group[])}
     * @rights membre d'un groupe author ou d'un groupe recipient
     * @async
     */
    async recipients(args, context: Context, info): Promise<Group[]> {
        let res = new Array<Group>();

        let groups = await context.models.message.getAnnouncementRecipients(this.mid);
        for(let g of groups) {
            res.push(await context.models.group.getGroup(g));
        }

        return res;
    }

    /**
     * @memberof GraphQL.Announcement#
     * @function views
     * @summary Renvoie le nombre de vues
     * @return {Promise(number)}
     * @rights membre d'un groupe author ou d'un groupe recipient
     * @async
     */
    async views(args, context: Context, info): Promise<number> {
        return this.getViews();
    }

    async getViews(): Promise<number> {
        await this.m_loader.load();
        return this.m_views;
    }

    /**
     * @memberof GraphQL.Announcement#
     * @function forEvent
     * @summary Renvoie l'evenement correspondant, s'il existe
     * @return {Promise(Event)}
     * @rights membre d'un groupe author ou d'un groupe recipient
     * @async
     */
    async forEvent(args, context: Context, info): Promise<Event> {
        return this.getForEvent();
    }

    async getForEvent(): Promise<Event> {
        let data = await knex.select('mid').from('messages_events').where('for_announcement', this.mid);
        if (data.length > 0) {
            return new Event(data[0].mid);
        }
        return null;
    }
}
