/**
 * @file Resolvers pour les réponses
 * @author akka, ofacklam
 */

import { Group } from '../groups/groups';
import { SimpleGroup } from "../groups/simpleGroups";
import { MetaGroup } from "../groups/metaGroups";

import { User } from '../users';

import { Message } from './messages';
import { Question } from './questions';

import { GroupSet } from '../../../utils/tools';
import { Context } from '../../typeDefs/queries';

import knex from '../../../../db/knex_router';
import { ApolloError } from 'apollo-server-core';

export class Answer extends Message {

    /**
     * @memberof GraphQL
     * @class Answer
     * @extends GraphQL.Message
     * @summary Resolvers des réponses
     * @classdesc Une classe représentant le type Answer du schéma.
     * Comme Apollo Server, par défaut, appelle la propriété / fonction avec le nom a résoudre, il n'y a pas besoin d'écrire les resolvers explicitement.
     * @arg {number} mid - Identifiant du message, supposé valide.
     * @rights viewer du groupe author
     */
    constructor(mid: number) {
        super(mid);
    }

    /**
     * @memberof GraphQL.Answer#
     * @function tryCreate
     * @summary Fonction qui va essayer de créer la réponse correspondante
     * @arg {number} mid - Identifiant du message, sans hypothese sur la validité.
     * @returns {Promise(Answer)} - Renvoie la Answer créée, ou null en cas d'erreur.
     * @rights viewer du groupe author
     * @async
     * @static
     */
    static async tryCreate(mid: number): Promise<Answer> {
        let m = new Answer(mid);
        if (await m.m_loader.load()) {
            return m;
        }
        return null;
    }

    /**
     * @memberof GraphQL.Answer#
     * @function fetchData
     * @summary Fonction qui va chercher toutes les données sur ce post privé dans la BDD, si ce n'est pas déja fait.
     * @returns {Promise(boolean)} Renvoie true si le chargement est réussi.
     * @async
     * @protected
     */
    protected async fetchData(): Promise<boolean> {
        let data = await knex.select('created_at',
            'updated_at',
            'title',
            'content',
            'author',
            'for_question').from('messages_answers').where('mid', this.mid);

        if (data.length > 0) {
            let m = data[0];

            this.m_createdAt = m.created_at;
            this.m_updatedAt = m.updated_at;
            this.m_title = m.title;
            this.m_content = m.content;
            this.m_author = m.recipient;
            this.m_forQuestion = m.for_question;

            return true;
        }

        return false;
    }

    /**
     * Protected properties.
     * Ce sont tous les champs triviaux, ie qui peuvent etre récupérés en une seule requete a la BDD,
     * ce qui permet d'etre plus efficace.
     * La variable dataLoaded témoigne de si on est déja allés chercher les données.
     * ATTENTION. Ce ne sont PAS directement les resolvers, il FAUT donc qu'ils aient un nom DIFFÉRENT du nom des champs dans le schéma.
     */
    protected m_author: string
    protected m_forQuestion: number

    /**
     * Ci-dessous les resolvers a proprement parler.
     */

    __typename: string = "Answer";

    /**
     * @memberof GraphQL.Answer#
     * @function author
     * @summary Renvoie le groupe auteur
     * @return {Promise(Group)}
     * @rights viewer du groupe author
     * @async
     */
    async author(args, context: Context, info): Promise<Group> {
        return context.models.group.getGroup(await this.getAuthor());
    }

    async getAuthor(): Promise<string> {
        await this.m_loader.load();
        return this.m_author;
    }

    /**
     * @memberof GraphQL.Answer#
     * @function forQuestion
     * @summary Renvoie la question correspondante
     * @return {Promise(Question)}
     * @rights viewer du groupe author
     * @async
     */
    async forQuestion(args, context: Context, info): Promise<Question> {
       return this.getForQuestion();
    }

    async getForQuestion(): Promise<Question> {
        await this.m_loader.load();
        return new Question(this.m_forQuestion);
    }
}
