/**
 * @file Resolvers pour les évenements
 * @author akka, ofacklam
 */

import { Group } from '../groups/groups';
import { SimpleGroup } from "../groups/simpleGroups";
import { MetaGroup } from "../groups/metaGroups";

import { User } from '../users';

import { Message } from './messages';
import { Announcement } from './announcements';

import { GroupSet } from '../../../utils/tools';
import { Context } from '../../typeDefs/queries';

import knex from '../../../../db/knex_router';
import { ApolloError } from 'apollo-server-core';

export class Event extends Message {

    /**
     * @memberof GraphQL
     * @class Event
     * @extends GraphQL.Message
     * @summary Resolvers des events
     * @classdesc Une classe représentant le type Event du schéma.
     * Comme Apollo Server, par défaut, appelle la propriété / fonction avec le nom a résoudre, il n'y a pas besoin d'écrire les resolvers explicitement.
     * @arg {number} mid - Identifiant du message, supposé valide.
     * @rights membre d'un groupe author ou d'un groupe recipient
     */
    constructor(mid: number) {
        super(mid);
    }

    /**
     * @memberof GraphQL.Event#
     * @function tryCreate
     * @summary Fonction qui va essayer de créer l'evenement correspondant
     * @arg {number} mid - Identifiant du message, sans hypothese sur la validité.
     * @returns {Promise(Event)} - Renvoie le Event créé, ou null en cas d'erreur.
     * @rights membre d'un groupe author ou d'un groupe recipient
     * @async
     * @static
     */
    static async tryCreate(mid: number): Promise<Event> {
        let m = new Event(mid);
        if (await m.m_loader.load()) {
            return m;
        }
        return null;
    }

    /**
     * @memberof GraphQL.Event#
     * @function fetchData
     * @summary Fonction qui va chercher toutes les données sur cet event dans la BDD, si ce n'est pas déja fait.
     * @returns {Promise(boolean)} Renvoie true si le chargement est réussi.
     * @async
     * @protected
     */
    protected async fetchData(): Promise<boolean> {
        let data = await knex.select('created_at',
            'updated_at',
            'title',
            'content',
            'location',
            'start_time',
            'end_time',
            'for_announcement').from('messages_events').where('mid', this.mid);

        if (data.length > 0) {
            let m = data[0];

            this.m_createdAt = m.created_at;
            this.m_updatedAt = m.updated_at;
            this.m_title = m.title;
            this.m_content = m.content;
            this.m_location = m.location;
            this.m_startTime = m.start_time;
            this.m_endTime = m.end_time;
            this.m_forAnnouncement = m.for_announcement;

            return true;
        }

        return false;
    }

    /**
     * Protected properties.
     * Ce sont tous les champs triviaux, ie qui peuvent etre récupérés en une seule requete a la BDD,
     * ce qui permet d'etre plus efficace.
     * La variable dataLoaded témoigne de si on est déja allés chercher les données.
     * ATTENTION. Ce ne sont PAS directement les resolvers, il FAUT donc qu'ils aient un nom DIFFÉRENT du nom des champs dans le schéma.
     */
    protected m_location: string
    protected m_startTime: string
    protected m_endTime: string
    protected m_forAnnouncement: number


    /**
     * Ci-dessous les resolvers a proprement parler.
     */

    __typename: string = "Event";

    /**
     * @memberof GraphQL.Event#
     * @function authors
     * @summary Renvoie les groupes auteurs
     * @return {Promise(Group[])}
     * @rights membre d'un groupe author ou d'un groupe recipient
     * @async
     */
    async authors(args, context: Context, info): Promise<Group[]> {
        let res = new Array<Group>();

        let groups = await context.models.message.getEventAuthors(this.mid);
        for (let g of groups) {
            res.push(await context.models.group.getGroup(g));
        }

        return res;
    }

    /**
     * @memberof GraphQL.Event#
     * @function recipients
     * @summary Renvoie les groupes destinataires
     * @return {Promise(Group[])}
     * @rights membre d'un groupe author ou d'un groupe recipient
     * @async
     */
    async recipients(args, context: Context, info): Promise<Group[]> {
        let res = new Array<Group>();

        let groups = await context.models.message.getEventRecipients(this.mid);
        for (let g of groups) {
            res.push(await context.models.group.getGroup(g));
        }

        return res;
    }

    /**
     * @memberof GraphQL.Event#
     * @function location
     * @summary Renvoie l'endroit
     * @return {Promise(string)}
     * @rights membre d'un groupe author ou d'un groupe recipient
     * @async
     */
    async location(args, context: Context, info): Promise<string> {
       return this.getLocation();
    }
    
    async getLocation(): Promise<string> {
        await this.m_loader.load();
        return this.m_location;
    }

    /**
     * @memberof GraphQL.Event#
     * @function startTime
     * @summary Renvoie l'horaire de début
     * @return {Promise(string)}
     * @rights membre d'un groupe author ou d'un groupe recipient
     * @async
     */
    async startTime(args, context: Context, info): Promise<string> {
        return this.getStartTime();
    }

    async getStartTime(): Promise<string> {
        await this.m_loader.load();
        return this.m_startTime;
    }

    /**
     * @memberof GraphQL.Event#
     * @function endTime
     * @summary Renvoie l'horaire de fin
     * @return {Promise(string)}
     * @rights membre d'un groupe author ou d'un groupe recipient
     * @async
     */
    async endTime(args, context: Context, info): Promise<string> {
        return this.getEndTime();
    }

    async getEndTime(): Promise<string> {
        await this.m_loader.load();
        return this.m_endTime;
    }

    /**
     * @memberof GraphQL.Event#
     * @function participatingGroups
     * @summary Renvoie les groupes qui participent
     * @return {Promise(Group[])}
     * @rights membre d'un groupe author ou d'un groupe recipient
     * @async
     */
    async participatingGroups(args, context: Context, info): Promise<Group[]> {
        let res = new Array<Group>();

        let groups = await context.models.message.getParticipatingGroups(this.mid);
        for (let g of groups) {
            res.push(await context.models.group.getGroup(g));
        }

        return res;
    }

    /**
     * @memberof GraphQL.Event#
     * @function participatingUsers
     * @summary Renvoie les users qui participent
     * @return {Promise(User[])}
     * @rights membre d'un groupe author ou d'un groupe recipient
     * @async
     */
    async participatingUsers(args, context: Context, info): Promise<User[]> {
        return context.models.message.getParticipatingUsers(this.mid);
    }

    /**
     * @memberof GraphQL.Event#
     * @function forAnnouncement
     * @summary Renvoie l'announcement correspondant, s'il existe
     * @return {Promise(Announcement)}
     * @rights membre d'un groupe author ou d'un groupe recipient
     * @async
     */
    async forAnnouncement(args, context: Context, info): Promise<Announcement> {
        return this.getForAnnouncement();
    }

    async getForAnnouncement(): Promise<Announcement> {
        await this.m_loader.load();

        if (this.m_forAnnouncement) { //what is the default ?????
            return new Announcement(this.m_forAnnouncement);
        }
        
        return null;
    }
}
