/**
 * @file Resolvers pour les messages
 * @author akka, ofacklam
 */

import { Group } from '../groups/groups';
import { SimpleGroup } from "../groups/simpleGroups";
import { MetaGroup } from "../groups/metaGroups";

import {User} from '../users';

import { GroupSet } from '../../../utils/tools';
import { Context } from '../../typeDefs/queries';
import { DataLoader } from '../../../utils/dataLoader';

import knex from '../../../../db/knex_router';
import { ApolloError } from 'apollo-server-core';

export abstract class Message {

    /**
     * @memberof GraphQL
     * @class Message
     * @summary Resolvers des messages
     * @classdesc Une classe abstraite représentant l'interface Message du schéma.
     * Comme Apollo Server, par défaut, appelle la propriété / fonction avec le nom a résoudre, il n'y a pas besoin d'écrire les resolvers explicitement.
     * @arg {number} mid - Identifiant du message, supposé valide.
     * @rights dépend du type de message
     * @abstract
     */
    constructor(mid: number) {
        this.mid = mid;
        this.m_loader = new DataLoader(this.fetchData.bind(this));
    }

    /**
     * @memberof GraphQL.Message#
     * @function fetchData
     * @summary Fonction qui va chercher toutes les données sur ce message dans la BDD, si ce n'est pas déja fait.
     * @returns {Promise(boolean)} Renvoie true si le chargement est réussi.
     * @async
     * @protected
     * @abstract
     */
    protected abstract async fetchData(): Promise<boolean>

    /**
     * Protected properties.
     * Ce sont tous les champs triviaux, ie qui peuvent etre récupérés en une seule requete a la BDD,
     * ce qui permet d'etre plus efficace.
     * La variable dataLoaded témoigne de si on est déja allés chercher les données.
     * ATTENTION. Ce ne sont PAS directement les resolvers, il FAUT donc qu'ils aient un nom DIFFÉRENT du nom des champs dans le schéma.
     */
    protected m_loader: DataLoader
    
    protected m_createdAt: string
    protected m_updatedAt: string
    protected m_title: string
    protected m_content: string

    /**
     * Ci-dessous les resolvers a proprement parler.
     */

    __typename: string = "Bad message type";

    /** @rights same as message object */
    mid: number;

    /**
     * @memberof GraphQL.Message#
     * @function createdAt
     * @summary Renvoie la date de création
     * @return {Promise(string)}
     * @rights same as message object
     * @async
     */
    async createdAt(args, context: Context, info): Promise<string> {
        return this.getCreatedAt();
    }

    async getCreatedAt(): Promise<string> {
        await this.m_loader.load();
        return this.m_createdAt;
    }

    /**
     * @memberof GraphQL.Message#
     * @function updatedAt
     * @summary Renvoie la date de mise a jour
     * @return {Promise(string)}
     * @rights same as message object
     * @async
     */
    async updatedAt(args, context: Context, info): Promise<string> {
        return this.getUpdatedAt();
    }

    async getUpdatedAt(): Promise<string> {
        await this.m_loader.load();
        return this.m_updatedAt;
    }

    /**
     * @memberof GraphQL.Message#
     * @function title
     * @summary Renvoie le titre du message
     * @return {Promise(string)}
     * @rights same as message object
     * @async
     */
    async title(args, context: Context, info): Promise<string> {
        return this.getTitle();
    }

    async getTitle(): Promise<string> {
        await this.m_loader.load();
        return this.m_title;
    }

    /**
     * @memberof GraphQL.Message#
     * @function content
     * @summary Renvoie le contenu du message
     * @return {Promise(string)}
     * @rights same as message object
     * @async
     */
    async content(args, context: Context, info): Promise<string> {
        return this.getContent();
    }

    async getContent(): Promise<string> {
        await this.m_loader.load();
        return this.m_content;
    }
}

