/**
 * @file Resolvers pour les posts privés
 * @author akka, ofacklam
 */

import { Group } from '../groups/groups';
import { SimpleGroup } from "../groups/simpleGroups";
import { MetaGroup } from "../groups/metaGroups";

import { User } from '../users';

import { Message } from './messages';

import { GroupSet } from '../../../utils/tools';
import { Context } from '../../typeDefs/queries';

import knex from '../../../../db/knex_router';
import { ApolloError } from 'apollo-server-core';

export class PrivatePost extends Message {

    /**
     * @memberof GraphQL
     * @class PrivatePost
     * @extends GraphQL.Message
     * @summary Resolvers des posts privés
     * @classdesc Une classe représentant le type PrivatePost du schéma.
     * Comme Apollo Server, par défaut, appelle la propriété / fonction avec le nom a résoudre, il n'y a pas besoin d'écrire les resolvers explicitement.
     * @arg {number} mid - Identifiant du message, supposé valide.
     * @rights membre du groupe recipient
     */
    constructor(mid: number) {
        super(mid);
    }

    /**
     * @memberof GraphQL.PrivatePost#
     * @function tryCreate
     * @summary Fonction qui va essayer de créer le post privé correspondant
     * @arg {number} mid - Identifiant du message, sans hypothese sur la validité.
     * @returns {Promise(PrivatePost)} - Renvoie le PrivatePost créé, ou null en cas d'erreur.
     * @rights membre du groupe recipient
     * @async
     * @static
     */
    static async tryCreate(mid: number): Promise<PrivatePost> {
        let m = new PrivatePost(mid);
        if (await m.m_loader.load()) {
            return m;
        }
        return null;
    }

    /**
     * @memberof GraphQL.PrivatePost#
     * @function fetchData
     * @summary Fonction qui va chercher toutes les données sur ce post privé dans la BDD, si ce n'est pas déja fait.
     * @returns {Promise(boolean)} Renvoie true si le chargement est réussi.
     * @async
     * @protected
     */
    protected async fetchData(): Promise<boolean> {
        let data = await knex.select('created_at',
            'updated_at',
            'title',
            'content',
            'author',
            'recipient').from('messages_privateposts').where('mid', this.mid);

        if (data.length > 0) {
            let m = data[0];

            this.m_createdAt = m.created_at;
            this.m_updatedAt = m.updated_at;
            this.m_title = m.title;
            this.m_content = m.content;
            this.m_author = m.author;
            this.m_recipient = m.recipient;

            return true;
        }
        
        return false;
    }

    /**
     * Protected properties.
     * Ce sont tous les champs triviaux, ie qui peuvent etre récupérés en une seule requete a la BDD,
     * ce qui permet d'etre plus efficace.
     * La variable dataLoaded témoigne de si on est déja allés chercher les données.
     * ATTENTION. Ce ne sont PAS directement les resolvers, il FAUT donc qu'ils aient un nom DIFFÉRENT du nom des champs dans le schéma.
     */
    protected m_author: string
    protected m_recipient: string

    /**
     * Ci-dessous les resolvers a proprement parler.
     */

    __typename: string = "PrivatePost";

    /**
     * @memberof GraphQL.PrivatePost#
     * @function author
     * @summary Renvoie le user auteur
     * @return {Promise(User)}
     * @rights membre du groupe recipient
     * @async
     */
    async author(args, context: Context, info): Promise<User> {
        return this.getAuthor();
    }

    async getAuthor(): Promise<User> {
        await this.m_loader.load();
        return new User(this.m_author);
    }


    /**
     * @memberof GraphQL.PrivatePost#
     * @function recipient
     * @summary Renvoie le groupe destinataire
     * @return {Promise(Group)}
     * @rights membre du groupe recipient
     * @async
     */
    async recipient(args, context: Context, info): Promise<Group> {
        return context.models.group.getGroup(await this.getRecipient());
    }

    async getRecipient() : Promise<string> {
        await this.m_loader.load();
        return this.m_recipient;
    }
}
