/**
 * @file Resolvers pour les questions
 * @author akka, ofacklam
 */

import { Group } from '../groups/groups';
import { SimpleGroup } from "../groups/simpleGroups";
import { MetaGroup } from "../groups/metaGroups";

import { User } from '../users';

import { Message } from './messages';
import { Answer } from './answers';

import { GroupSet } from '../../../utils/tools';
import { Context } from '../../typeDefs/queries';

import knex from '../../../../db/knex_router';
import { ApolloError } from 'apollo-server-core';

export class Question extends Message {

    /**
     * @memberof GraphQL
     * @class Question
     * @extends GraphQL.Message
     * @summary Resolvers des questions
     * @classdesc Une classe représentant le type Question du schéma.
     * Comme Apollo Server, par défaut, appelle la propriété / fonction avec le nom a résoudre, il n'y a pas besoin d'écrire les resolvers explicitement.
     * @arg {number} mid - Identifiant du message, supposé valide.
     * @rights viewer du groupe recipient
     */
    constructor(mid: number) {
        super(mid);
    }

    /**
     * @memberof GraphQL.Question#
     * @function tryCreate
     * @summary Fonction qui va essayer de créer la question correspondante
     * @arg {number} mid - Identifiant du message, sans hypothese sur la validité.
     * @returns {Promise(Question)} - Renvoie la Question créée, ou null en cas d'erreur.
     * @rights viewer du groupe recipient
     * @async
     * @static
     */
    static async tryCreate(mid: number): Promise<Question> {
        let m = new Question(mid);
        if (await m.m_loader.load()) {
            return m;
        }
        return null;
    }

    /**
     * @memberof GraphQL.Question#
     * @function fetchData
     * @summary Fonction qui va chercher toutes les données sur ce post privé dans la BDD, si ce n'est pas déja fait.
     * @returns {Promise(boolean)} Renvoie true si le chargement est réussi.
     * @async
     * @protected
     */
    protected async fetchData(): Promise<boolean> {
        let data = await knex.select('created_at',
            'updated_at',
            'title',
            'content',
            'author',
            'recipient',
            'for_answer').from('messages_questions').where('mid', this.mid);

        if (data.length > 0) {
            let m = data[0];

            this.m_createdAt = m.created_at;
            this.m_updatedAt = m.updated_at;
            this.m_title = m.title;
            this.m_content = m.content;
            this.m_author = m.author;
            this.m_recipient = m.recipient;
            this.m_forAnswer = m.for_answer;

            return true;
        }
        
        return false;
    }

    /**
     * Protected properties.
     * Ce sont tous les champs triviaux, ie qui peuvent etre récupérés en une seule requete a la BDD,
     * ce qui permet d'etre plus efficace.
     * La variable dataLoaded témoigne de si on est déja allés chercher les données.
     * ATTENTION. Ce ne sont PAS directement les resolvers, il FAUT donc qu'ils aient un nom DIFFÉRENT du nom des champs dans le schéma.
     */
    protected m_author: string
    protected m_recipient: string
    protected m_forAnswer: number

    /**
     * Ci-dessous les resolvers a proprement parler.
     */

    __typename: string = "Question";

    /**
     * @memberof GraphQL.Question#
     * @function author
     * @summary Renvoie le user auteur
     * @return {Promise(User)}
     * @rights viewer du groupe recipient
     * @async
     */
    async author(args, context: Context, info): Promise<User> {
        return this.getAuthor();
    }

    async getAuthor(): Promise<User> {
        await this.m_loader.load();
        return new User(this.m_author);
    }

    /**
     * @memberof GraphQL.Question#
     * @function recipient
     * @summary Renvoie le groupe destinataire
     * @return {Promise(Group)}
     * @rights viewer du groupe recipient
     * @async
     */
    async recipient(args, context: Context, info): Promise<Group> {
        return context.models.group.getGroup(await this.getRecipient());
    }

    async getRecipient(): Promise<string> {
        await this.m_loader.load();
        return this.m_recipient;
    }

    /**
     * @memberof GraphQL.Question#
     * @function forAnswer
     * @summary Renvoie la réponse correspondante, si elle existe
     * @return {Promise(Answer)}
     * @rights viewer du groupe recipient
     * @async
     */
    async forAnswer(args, context: Context, info): Promise<Answer> {
        let data = await knex.select('mid').from('messages_answers').where('for_question', this.mid);
        if (data.length > 0) {
            return new Answer(data[0].mid);
        }
        return null;
    }
}
