/**
 * @file Resolvers pour les requetes GroupCoauthorEvent
 * @author akka, ofacklam
 */

import { Group } from "../groups/groups";
import { SimpleGroup } from "../groups/simpleGroups";
import { MetaGroup } from "../groups/metaGroups";

import { User } from '../users';

import { Event } from "../messages/events";

import { Request } from './requests'

import knex from '../../../../db/knex_router';
import { Context } from "../../typeDefs/queries";
import { ApolloError } from "apollo-server-core";


export class GroupCoauthorEvent extends Request {

    /**
     * @memberof GraphQL
     * @class GroupCoauthorEvent
     * @extends GraphQL.Request
     * @summary Resolvers des requetes GroupCoauthorEvent
     * @classdesc Une classe représentant le type GroupCoauthorEvent du schéma.
     * Comme Apollo Server, par défaut, appelle la propriété / fonction avec le nom a résoudre, il n'y a pas besoin d'écrire les resolvers explicitement.
     * @arg {number} rid - Identifiant de la requete, supposé valide.
     * @rights admin du groupe émetteur ou destinataire
     */
    constructor(rid: number) {
        super(rid);
    }

    /**
     * @memberof GraphQL.GroupCoauthorEvent#
     * @function tryCreate
     * @summary Fonction qui va essayer de créer la requete correspondante
     * @arg {number} rid - Identifiant de la requete, sans hypothese sur la validité.
     * @returns {Promise(GroupCoauthorEvent)} - Renvoie le GroupCoauthorEvent créé, ou null en cas d'erreur.
     * @rights admin du groupe émetteur ou destinataire
     * @async
     * @static
     */
    static async tryCreate(rid: number): Promise<GroupCoauthorEvent> {
        let r = new GroupCoauthorEvent(rid);
        if (await r.m_loader.load()) {
            return r;
        }
        return null;
    }

    /**
     * @memberof GraphQL.GroupCoauthorEvent#
     * @function fetchData
     * @summary Fonction qui va chercher toutes les données sur cette requete dans la BDD, si ce n'est pas déja fait.
     * @returns {Promise(boolean)} Renvoie true si le chargement est réussi.
     * @async
     * @protected
     */
    protected async fetchData(): Promise<boolean> {
        let data = await knex.select('created_at',
            'comment',
            'from',
            'to',
            'for_event').from('requests_group_coauthor_event').where('rid', this.rid);

        if (data.length > 0) {
            let r = data[0];

            this.createdAt = r.created_at;
            this.m_comment = r.comment;
            this.m_from = r.from;
            this.m_to = r.to;
            this.m_forEvent = r.for_event;

            return true;
        }
        
        return false;
    }

    /**
     * Protected properties.
     * Ce sont tous les champs triviaux, ie qui peuvent etre récupérés en une seule requete a la BDD,
     * ce qui permet d'etre plus efficace.
     * La variable dataLoaded témoigne de si on est déja allés chercher les données.
     * ATTENTION. Ce ne sont PAS directement les resolvers, il FAUT donc qu'ils aient un nom DIFFÉRENT du nom des champs dans le schéma.
     */
    protected m_from: string;
    protected m_forEvent: number;

    /**
     * Ci-dessous les resolvers a proprement parler.
     */

    __typename: string = "GroupCoauthorEvent";

    /**
     * @memberof GraphQL.GroupCoauthorEvent#
     * @function from
     * @summary Renvoie le groupe émetteur
     * @return {Promise(Group)}
     * @rights admin du groupe émetteur ou destinataire
     * @async
     */
    async from(args, context: Context, info): Promise<Group> {
        return context.models.group.getGroup(await this.getFrom());
    }

    async getFrom() : Promise<string> {
        await this.m_loader.load();
        return this.m_from;
    }

    /**
     * @memberof GraphQL.GroupCoauthorEvent#
     * @function forEvent
     * @summary Renvoie l'evenement a co-organiser
     * @return {Promise(Event)}
     * @rights admin du groupe émetteur ou destinataire
     * @async
     */
    async forEvent(args, context: Context, info): Promise<Event> {
        await this.m_loader.load();
        throw "Not implemented";
        //a-t-il le droit de voir ???
        // On suppose que s'il a accès à la requête, c'est que :
        // - Soit il est dans le groupe organisateur : RAS
        // - Soit il l'a postée, et donc avait le droit de voir l'event : RAS
        // - Soit un autre admin du groupe émetteur a posté la requête, mais tous les admins n'ont pas forcément le droit de voir l'event...
        return new Event(this.m_forEvent);
    }
}