/**
 * @file Resolvers pour tous les requetes GroupJoinMetagroup
 * @author akka, ofacklam
 */

import { Group } from "../groups/groups";
import { SimpleGroup } from "../groups/simpleGroups";
import { MetaGroup } from "../groups/metaGroups";

import { User } from '../users';

import { Event } from "../messages/events";

import { Request } from './requests'

import knex from '../../../../db/knex_router';
import { Context } from "../../typeDefs/queries";
import { ApolloError } from "apollo-server-core";


export class GroupJoinMetagroup extends Request {

    /**
     * @memberof GraphQL
     * @class GroupJoinMetagroup
     * @extends GraphQL.Request
     * @summary Resolvers des requetes GroupJoinMetagroup
     * @classdesc Une classe représentant le type GroupJoinMetagroup du schéma.
     * Comme Apollo Server, par défaut, appelle la propriété / fonction avec le nom a résoudre, il n'y a pas besoin d'écrire les resolvers explicitement.
     * @arg {number} rid - Identifiant de la requete, supposé valide.
     * @rights admin du groupe émetteur ou destinataire
     */
    constructor(rid: number) {
        super(rid);
    }

    /**
     * @memberof GraphQL.GroupJoinMetagroup#
     * @function tryCreate
     * @summary Fonction qui va essayer de créer la requete correspondante
     * @arg {number} rid - Identifiant de la requete, sans hypothese sur la validité.
     * @returns {Promise(GroupJoinMetagroup)} - Renvoie le GroupJoinMetagroup créé, ou null en cas d'erreur.
     * @rights admin du groupe émetteur ou destinataire
     * @async
     * @static
     */
    static async tryCreate(rid: number): Promise<GroupJoinMetagroup> {
        let r = new GroupJoinMetagroup(rid);
        if (await r.m_loader.load()) {
            return r;
        }
        return null;
    }

    /**
     * @memberof GraphQL.GroupJoinMetagroup#
     * @function fetchData
     * @summary Fonction qui va chercher toutes les données sur cette requete dans la BDD, si ce n'est pas déja fait.
     * @returns {Promise(boolean)} Renvoie true si le chargement est réussi.
     * @async
     * @protected
     */
    protected async fetchData(): Promise<boolean> {
        let data = await knex.select('created_at',
            'comment',
            'from',
            'to').from('requests_group_join_metagroup').where('rid', this.rid);

        if (data.length > 0) {
            let r = data[0];

            this.m_createdAt = r.created_at;
            this.m_comment = r.comment;
            this.m_from = r.from;
            this.m_to = r.to;

            return true;
        }
        
        return false;
    }

    /**
     * Protected properties.
     * Ce sont tous les champs triviaux, ie qui peuvent etre récupérés en une seule requete a la BDD,
     * ce qui permet d'etre plus efficace.
     * La variable dataLoaded témoigne de si on est déja allés chercher les données.
     * ATTENTION. Ce ne sont PAS directement les resolvers, il FAUT donc qu'ils aient un nom DIFFÉRENT du nom des champs dans le schéma.
     */
    protected m_from: string;

    /**
     * Ci-dessous les resolvers a proprement parler.
     */

    __typename: string = "GroupJoinMetagroup";

    /**
     * @memberof GraphQL.GroupJoinMetagroup#
     * @function from
     * @summary Renvoie le groupe émetteur
     * @return {Promise(SimpleGroup)}
     * @rights admin du groupe émetteur ou destinataire
     * @async
     */
    async from(args, context: Context, info): Promise<SimpleGroup> {
        return this.getFrom();    
    }

    async getFrom() : Promise<SimpleGroup> {
        await this.m_loader.load();
        return new SimpleGroup(this.m_from);
    }

}