/**
 * @file Resolvers pour le type requete
 * @author akka, ofacklam
 */

import { Group } from "../groups/groups";
import { SimpleGroup } from "../groups/simpleGroups";
import { MetaGroup } from "../groups/metaGroups";

import { User } from '../users';

import { Event } from "../messages/events";

import { UserJoinGroup } from './userJoinGroup';
import { GroupJoinMetagroup } from './groupJoinMetagroup';
import { GroupCoauthorEvent } from './groupCoauthorEvent';

import knex from '../../../../db/knex_router';
import { Context } from "../../typeDefs/queries";
import { DataLoader } from '../../../utils/dataLoader';

import { ApolloError } from "apollo-server-core";


export abstract class Request {

    /**
     * @memberof GraphQL
     * @class Request
     * @summary Resolvers des requetes
     * @classdesc Une classe abstraite représentant l'interface Request du schéma.
     * Comme Apollo Server, par défaut, appelle la propriété / fonction avec le nom a résoudre, il n'y a pas besoin d'écrire les resolvers explicitement.
     * @arg {number} rid - Identifiant de la requete, supposé valide.
     * @rights dépend du type de requete
     * @abstract
     */
    constructor(rid: number) {
        this.rid = rid;
        this.m_loader = new DataLoader(this.fetchData.bind(this));
    }

    /**
     * @memberof GraphQL.Request#
     * @function fetchData
     * @summary Fonction qui va chercher toutes les données sur cette requete dans la BDD, si ce n'est pas déja fait.
     * @returns {Promise(boolean)} Renvoie true si le chargement est réussi.
     * @async
     * @protected
     * @abstract
     */
    protected abstract async fetchData(): Promise<boolean>

    /**
     * Protected properties.
     * Ce sont tous les champs triviaux, ie qui peuvent etre récupérés en une seule requete a la BDD,
     * ce qui permet d'etre plus efficace.
     * La variable dataLoaded témoigne de si on est déja allés chercher les données.
     * ATTENTION. Ce ne sont PAS directement les resolvers, il FAUT donc qu'ils aient un nom DIFFÉRENT du nom des champs dans le schéma.
     */
    protected m_loader: DataLoader

    protected m_createdAt: string
    protected m_comment: string
    protected m_to: string

    /**
     * Ci-dessous les resolvers a proprement parler.
     */

    __typename: string = "Bad request type";

    /** @rights same as request object */
    rid: number;

    /**
     * @memberof GraphQL.Request#
     * @function createdAt
     * @summary Renvoie la date de création de la requete
     * @return {Promise(string)}
     * @rights same as request object
     * @async
     */
    async createdAt(args, context: Context, info): Promise<string> {
        return this.getCreatedAt();
    }

    async getCreatedAt(): Promise<string> {
        await this.m_loader.load();
        return this.m_createdAt;
    }

    /**
     * @memberof GraphQL.Request#
     * @function comment
     * @summary Renvoie le commentaire associé
     * @return {Promise(string)}
     * @rights same as request object
     * @async
     */
    async comment(args, context: Context, info): Promise<string> {
        return this.getComment();
    }

    async getComment(): Promise<string> {
        await this.m_loader.load();
        return this.m_comment;
    }

    /**
     * @memberof GraphQL.Request#
     * @function to
     * @summary Renvoie le destinataire de la demande
     * @return {Promise(Group)}
     * @rights same as request object
     * @async
     */
    async to(args, context: Context, info): Promise<Group> {
        return context.models.group.getGroup(await this.getTo());
    }

    async getTo(): Promise<string> {
        await this.m_loader.load();
        return this.m_to;
    }
}

