/**
 * @file Resolvers des utilisateurs
 * @author ofacklam
 */

import { Group } from './groups/groups';
import { SimpleGroup } from "./groups/simpleGroups";
import { MetaGroup } from "./groups/metaGroups";

import {Question} from './messages/questions';

import {User as LDAP_User} from '../../ldap/export/user';
import {Group as LDAP_Group} from '../../ldap/export/group';

import { DataLoader } from '../../utils/dataLoader';
import {Tools, GroupSet} from '../../utils/tools';
import { Context } from '../typeDefs/queries';

export class User {

    /**
     * @memberof GraphQL
     * @class User
     * @summary Resolvers des utilisateurs
     * @classdesc Une classe représentant le type User du schéma.
     * Comme Apollo Server, par défaut, appelle la propriété / fonction avec le nom a résoudre, il n'y a pas besoin d'écrire les resolvers explicitement.
     * @arg {string} uid - Identifiant du user, supposé valide.
     * @rights connectedOrOnplatal
     */
    constructor(uid: string) {
        this.uid = uid;
        this.m_loader = new DataLoader(this.fetchData.bind(this));
    }

    /**
     * @memberof GraphQL.User#
     * @function tryCreate
     * @summary Fonction qui va essayer de créer le user correspondant
     * @arg {string} uid - Identifiant du user, sans hypothese sur la validité.
     * @returns {Promise(User)} - Renvoie le user créé, ou null en cas d'erreur.
     * @rights connectedOrOnplatal
     * @async
     * @static
     */
    static async tryCreate(uid: string): Promise<User> {
        let u = new User(uid);
        if(await u.m_loader.load()) {
            return u;
        }
        return null;
    }

    /**
     * @memberof GraphQL.User#
     * @function fetchData
     * @summary Fonction qui va chercher toutes les données sur ce user dans le LDAP, si ce n'est pas déja fait.
     * @returns {Promise(boolean)} Renvoie true si le chargement est réussi.
     * @async
     * @protected
     */
    protected async fetchData(): Promise<boolean> {
        try {
            let data = await LDAP_User.peek(this.uid);
            if(data.uid !== this.uid) {
                throw "Error in search";
            }

            this.m_givenName = data.givenName;
            this.m_lastName = data.lastName;
            this.m_nickname = data.nickname;
            this.m_nationality = data.nationality;
            this.m_birthdate = data.birthdate;
            this.m_photo = data.urlPhoto;

            this.m_mail = data.mail;
            this.m_phone = data.phone;
            this.m_address = data.address;

            this.m_memberOf = data.members;
            this.m_speakerOf = data.speakers;
            this.m_adminOf = data.admins;
            this.m_likes = data.followers;

            return true;
        }
        catch {
            return false;
        }
    }

    /**
     * Protected properties.
     * Ce sont tous les champs triviaux, ie qui peuvent etre récupérés en une seule requete au LDAP,
     * ce qui permet d'etre plus efficace.
     * La variable dataLoaded témoigne de si on est déja allés chercher les données.
     * ATTENTION. Ce ne sont PAS directement les resolvers, il FAUT donc qu'ils aient un nom DIFFÉRENT du nom des champs dans le schéma.
     */
    protected m_loader: DataLoader

    protected m_givenName: string
    protected m_lastName: string
    protected m_nickname: string
    protected m_nationality: string
    protected m_birthdate: string
    protected m_photo: string

    protected m_mail: string
    protected m_phone: string
    protected m_address: string

    protected m_memberOf: string[]
    protected m_speakerOf: string[]
    protected m_adminOf: string[]
    protected m_likes: string[]

    /**
     * Ci-dessous les resolvers a proprement parler.
     */

    /** @rights connectedOrOnplatal */
    uid: string;

    /**
     * @memberof GraphQL.User#
     * @function givenName
     * @summary Renvoie le prénom
     * @return {Promise(string)}
     * @rights connectedOrOnplatal
     * @async
     */
    async givenName(args, context: Context, info): Promise<string> {
        return this.getGivenName();
    }

    async getGivenName() : Promise<string> {
        await this.m_loader.load();
        return this.m_givenName;
    }

    /**
     * @memberof GraphQL.User#
     * @function lastName
     * @summary Renvoie le nom de famille
     * @return {Promise(string)}
     * @rights connectedOrOnplatal
     * @async
     */
    async lastName(args, context: Context, info): Promise<string> {
        return this.getLastName();
    }

    async getLastName() : Promise<string> {
        await this.m_loader.load();
        return this.m_lastName;
    }

    /**
     * @memberof GraphQL.User#
     * @function nickname
     * @summary Renvoie le surnom
     * @return {Promise(string)}
     * @rights connectedOrOnplatal
     * @async
     */
    async nickname(args, context: Context, info): Promise<string> {
        return this.getNickname();
    }

    async getNickname() : Promise<string> {
        await this.m_loader.load();
        return this.m_nickname;
    }

    /**
     * @memberof GraphQL.User#
     * @function nationality
     * @summary Renvoie la nationalité
     * @return {Promise(string)}
     * @rights connectedOrOnplatal
     * @async
     */
    async nationality(args, context: Context, info): Promise<string> {
        return this.getNationality();
    }

    async getNationality(): Promise<string> {
        await this.m_loader.load();
        return this.m_nationality;
    }

    /**
     * @memberof GraphQL.User#
     * @function birthdate
     * @summary Renvoie la date de naissance
     * @return {Promise(string)}
     * @rights connectedOrOnplatal
     * @async
     */
    async birthdate(args, context: Context, info): Promise<string> {
        return this.getBirthdate();
    }

    async getBirthdate() : Promise<string> {
        await this.m_loader.load();
        return this.m_birthdate;
    }

    /**
     * @memberof GraphQL.User#
     * @function photo
     * @summary Renvoie l'URL de la photo
     * @return {Promise(string)}
     * @rights connectedOrOnplatal
     * @async
     */
    async photo(args, context: Context, info): Promise<string> {
        return this.getPhoto();
    }

    async getPhoto(): Promise<string> {
        await this.m_loader.load();
        return this.m_photo;
    }

    /**
     * @memberof GraphQL.User#
     * @function mail
     * @summary Renvoie l'adresse mail
     * @return {Promise(string)}
     * @rights connectedOrOnplatal
     * @async
     */
    async mail(args, context: Context, info): Promise<string> {
        return this.getMail();
    }

    async getMail() : Promise<string> {
        await this.m_loader.load();
        return this.m_mail;
    }

    /**
     * @memberof GraphQL.User#
     * @function phone
     * @summary Renvoie le numéro de téléphone
     * @return {Promise(string)}
     * @rights connectedOrOnplatal
     * @async
     */
    async phone(args, context: Context, info): Promise<string> {
        return this.getPhone();
    }

    async getPhone(): Promise<string> {
        await this.m_loader.load();
        return this.m_phone;
    }

    /**
     * @memberof GraphQL.User#
     * @function address
     * @summary Renvoie l'adresse
     * @return {Promise(string)}
     * @rights connectedOrOnplatal
     * @async
     */
    async address(args, context: Context, info): Promise<string> {
        return this.getAddress();
    }

    async getAddress(): Promise<string> {
        await this.m_loader.load();
        return this.m_address;
    }

    /**
     * @memberof GraphQL.User#
     * @function memberOf
     * @summary Renvoie les groupes dont il est membre
     * @return {Promise(Group[])}
     * @rights connectedOrOnplatal
     * @async
     */
    async memberOf(args, context: Context, info): Promise<Group[]> {
        throw "Not implemented";
    }

    /**
     * @memberof GraphQL.User#
     * @function inheritedMemberOf
     * @summary Renvoie les groupes dont il est membre hérité
     * @return {Promise(Group[])}
     * @rights connectedOrOnplatal
     * @async
     */
    async inheritedMemberOf(args, context: Context, info): Promise<Group[]> {
        return this.getInheritedMemberOf();
    }

    async getInheritedMemberOf(): Promise<Group[]> {
        await this.m_loader.load();

        let simple = new GroupSet(this.m_memberOf);
        let meta = await Tools.metaGroupsOfGroups(simple);

        let all = new Array<Group>();

        for (let gid of simple) {
            all.push(new SimpleGroup(gid));
        }
        for (let gid of meta) {
            all.push(new MetaGroup(gid));
        }

        return all;
    }

    /**
     * @memberof GraphQL.User#
     * @function speakerOf
     * @summary Renvoie les groupes dont il est speaker
     * @return {Promise(Group[])}
     * @rights connectedOrOnplatal
     * @async
     */
    async speakerOf(args, context: Context, info): Promise<Group[]> {
        return this.getSpeakerOf();
    }

    async getSpeakerOf(): Promise<Group[]> {
        await this.m_loader.load();

        let speaker = new GroupSet(this.m_speakerOf);
        let meta = await Tools.metaGroupsOfGroups(speaker);

        let all = new Array<Group>();

        for(let gid of speaker) {
            all.push(new SimpleGroup(gid));
        }
        for(let gid of meta) {
            all.push(new MetaGroup(gid));
        }

        return all;
    }

    /**
     * @memberof GraphQL.User#
     * @function adminOf
     * @summary Renvoie les groupes dont il est admin
     * @return {Promise(Group[])}
     * @rights connectedOrOnplatal
     * @async
     */
    async adminOf(args, context: Context, info): Promise<Group[]> {
        throw "Not implemented";
    }

    /**
     * @memberof GraphQL.User#
     * @function inheritedAdminOf
     * @summary Renvoie les groupes dont il est admin hérité
     * @return {Promise(Group[])}
     * @rights connectedOrOnplatal
     * @async
     */
    async inheritedAdminOf(args, context: Context, info): Promise<Group[]> {
        return this.getInheritedAdminOf();
    }

    async getInheritedAdminOf(): Promise<Group[]> {
        await this.m_loader.load();

        let simple = new GroupSet(this.m_adminOf);
        let meta = await Tools.metaGroupsOfGroups(simple);

        let all = new Array<Group>();

        for (let gid of simple) {
            all.push(new SimpleGroup(gid));
        }
        for (let gid of meta) {
            all.push(new MetaGroup(gid));
        }

        return all;
    }

    /**
     * @memberof GraphQL.User#
     * @function likes
     * @summary Renvoie les groupes dont il est sympathisant
     * @return {Promise(Group[])}
     * @rights connectedOrOnplatal
     * @async
     */
    async likes(args, context: Context, info): Promise<Group[]> {
        return this.getLikes();
    }

    async getLikes(): Promise<Group[]> {
        await this.m_loader.load();

        let simple = new GroupSet(this.m_likes);
        let meta = await Tools.metaGroupsOfGroups(simple);

        let all: Group[];

        for (let gid of simple) {
            all.push(new SimpleGroup(gid));
        }
        for (let gid of meta) {
            all.push(new MetaGroup(gid));
        }

        return all;
    }

    /**
     * @memberof GraphQL.User#
     * @function dislikes
     * @summary Renvoie les groupes qu'il veut cacher
     * @return {Promise(Group[])}
     * @rights connectedOrOnplatal
     * @async
     */
    async dislikes(args, context: Context, info): Promise<Group[]> {
        throw "Not implemented";
    }

    /**
     * @memberof GraphQL.User#
     * @function questionsFromUser
     * @summary Renvoie les questions que le user a posées
     * @return {Promise(Question[])}
     * @rights viewer of recipient group
     * @async
     */
    async questionsFromUser(args, context: Context, info): Promise<Question[]> {
        await this.m_loader.load();
        throw "Not implemented";
    }
 }
